#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>

main (int argc, char **argv) {
  char buf[PATH_MAX];
  pid_t pid;
  if ( argc < 2 ) {
    fprintf(stderr,"%s: Requires at least one argument\n", argv[0]);
    exit(-1);
  }

  if ((pid = fork()) < 0) {
    snprintf(buf, sizeof(buf), "%s: Couldn't fork",argv[0]);
    perror(buf);
    exit(-1);
  } else if (pid > 0) {
    /* Parent */
    int status;
    int ret;

    while (1) {
      if ((ret = waitpid(pid, &status, WUNTRACED)) < 0 && errno == EINTR)
        continue;
      else if (ret < 0) {
        perror("waitpid failed");
        exit(1);
      }

      if (!WIFSTOPPED(status))
	break;

      /* Reset the handler in case it was inherited ignored,
	 but the child reset it and stopped anyway.  */
      signal(WSTOPSIG(status), SIG_DFL);
      raise(WSTOPSIG(status));

      /* Now we stop until continued ourselves.  */
      kill(getpgid(pid) == pid ? -pid : pid, SIGCONT);
    }

    if (WIFEXITED(status))
      exit(WEXITSTATUS(status));
    else if (WIFSIGNALED(status))
      /* XXX print here like the shell would? */
      exit(128 + WTERMSIG(status));
    else
      exit(127);		/* Should never happen.  */
  } else {
    /* Child */
    execv(argv[1], &argv[1]);

    snprintf(buf, sizeof(buf), "%s: Error execing %s", argv[0], argv[1]);
    perror(buf);
    exit(-1);
  }
}
